// -*- C++ -*-
#include "Rivet/Analysis.hh"
#include "Rivet/RivetAIDA.hh"
#include "Rivet/Tools/Logging.hh"
#include "Rivet/Projections/FinalState.hh"
#include "Rivet/Projections/ZFinder.hh"
#include "Rivet/Projections/FastJets.hh"
#include "Rivet/Tools/ParticleIdUtils.hh"


namespace Rivet {

  class ATLAS_2014_I1306294 : public Analysis {
  public:

    /// @name Constructors etc.
    //@{

    /// Constructors
    ATLAS_2014_I1306294(std::string name="ATLAS_2014_I1306294") 
      : Analysis(name)
    {
      _mode = 1;
      setNeedsCrossSection(true);
    }

    //@}

  public:

    /// @name Analysis methods
    //@{

    /// Book histograms and initialise projections before the run
    void init() {

      if (_mode==1) {
	//ee mode
	ZFinder zfinder(-2.5, 2.5, 20.0*GeV, ELECTRON, 76.0*GeV, 106.0*GeV, 0.1, true, true);
	addProjection(zfinder, "ZFinder");
      }
      else if (_mode==2) {
	//mumu mode
	ZFinder zfinder(-2.5, 2.5, 20.0*GeV, MUON, 76.0*GeV, 106.0*GeV, 0.1, true, true);
	addProjection(zfinder, "ZFinder");
      }

      FastJets jetpro1( getProjection<ZFinder>("ZFinder").remainingFinalState(), FastJets::ANTIKT, 0.4);
      jetpro1.useInvisibles();
      addProjection(jetpro1, "AntiKtJets04");
      
      //list of weakly decaying B hadrons
      //http://pdg.lbl.gov/mc_particle_id_contents.html
      stableB.push_back(511); //B0
      stableB.push_back(521); //B
      stableB.push_back(531); //Bs
      stableB.push_back(541); //Bc
      stableB.push_back(5122); //lambda0_b
      //  stableB.push_back(5112); //sigma_b -> decays to lambdab pi
      //  stableB.push_back(5212); //sigma0_b -> decays to lambda0b pi
      //  stableB.push_back(5222); //sigma+_b -> decays to lambdab pi
      stableB.push_back(5232); //xi0_b
      stableB.push_back(5132); //xi_b
      stableB.push_back(5142); //xi0_bc
      stableB.push_back(5242); //xi_bc
      stableB.push_back(5332); //omega_b
      
            
      //Histograms with data binning
      _h_bjet_Pt      = bookHistogram1D(3, 1, 1);
      _h_bjet_Y       = bookHistogram1D(5, 1, 1);
      _h_bjet_Yboost  = bookHistogram1D(7, 1, 1);
      _h_bjet_DY20    = bookHistogram1D(9, 1, 1);
      _h_bjet_ZdPhi20 = bookHistogram1D(11, 1, 1);
      _h_bjet_ZdR20   = bookHistogram1D(13, 1, 1);
      _h_bjet_ZPt     = bookHistogram1D(15, 1, 1);
      _h_bjet_ZY      = bookHistogram1D(17, 1, 1);
      _h_2bjet_dR     = bookHistogram1D(21, 1, 1);
      _h_2bjet_Mbb    = bookHistogram1D(23, 1, 1);
      _h_2bjet_ZPt    = bookHistogram1D(25, 1, 1);
      _h_2bjet_ZY     = bookHistogram1D(27, 1, 1);
    }

	  //==========================================================================================


    /// Perform the per-event analysis
    void analyze(const Event& e) {

      
      //---------------------------
      const double weight = e.weight();
      
      // -- check we have a Z: 
      const ZFinder& zfinder = applyProjection<ZFinder>(e, "ZFinder");
      
      if (zfinder.bosons().size() != 1) {
	vetoEvent;
      }
      
      const ParticleVector boson_s =  zfinder.bosons();
      const Particle       boson_f =  boson_s[0]      ;
      const ParticleVector zleps   =  zfinder.constituents();
      //---------------------------
      
      
      //---------------------------
      // -- look for b's
      bool hasB = false;
      int ipart = 0; // counter of particles 
      ParticleVector stableBs;
      foreach (const GenParticle* p, particles(e.genEvent())) {
	++ipart; 			
	
	//cut on pT:
	if(p->momentum().perp() < 5) continue;	
	
	for(uint k=0; k < stableB.size(); k++) {
	  if (fabs(p->pdg_id()) == stableB[k]) {
	    // would need to check if it has B daughters
	    stableBs += Particle(*p);
	    hasB = true;
	  }
	}  	
      }
      
      
      //------------- stop processing the event if no true b-partons or hadrons are found
      if ( ! hasB) {
	vetoEvent;
      }
      
      //---------------------------
      // -- get the b-jets:
      const FastJets& jetpro04 = applyProjection<FastJets>(e, "AntiKtJets04");
      // jets are ordered in pT, with a rapidity cut: 
      const Jets& jets = jetpro04.jetsByPt(20.0*GeV, 14000*GeV, -2.4, 2.4, RAPIDITY);
      
      std::vector<Rivet::Jet> b_jets;
      b_jets.clear();
      float dR_match = 0.3;
      
      foreach (const Jet& jet, jets) {
	
	//veto overlaps with Z leptons:
	bool veto = false;
	foreach (const Particle& zlep, zleps) {
	  if (deltaR(jet.momentum().eta(), jet.momentum().phi(), zlep.momentum().eta(), zlep.momentum().phi()) < 0.5) 
	    veto = true;
	}
	if (veto) continue;
	
	bool matchB = false;
	foreach (const Particle& bhadron,  stableBs) {

	  const float dR = deltaR(jet.momentum().eta(), jet.momentum().phi(), 
				  bhadron.momentum().eta(),bhadron.momentum().phi());
	  
	  if ( dR <= dR_match) {
	    matchB = true;
	  }//match!
	} // end loop on b-hadrons  
	
	if(matchB) b_jets.push_back(jet);
      }
      
      //and make sure we have at least 1:
      if (b_jets.size() < 1) {
	vetoEvent;
      }
      
      //---------------------------
      // fill the plots:
      const double ZpT = boson_f.momentum().pT()/GeV;
      const double ZY  = fabs(boson_f.momentum().rapidity());
      
      _h_bjet_ZPt->fill(ZpT, weight);
      _h_bjet_ZY ->fill(ZY,  weight);
      
      foreach (const Jet& jet, b_jets) {
	
	_h_bjet_Pt->fill(jet.momentum().pT()/GeV, weight );
	_h_bjet_Y ->fill(fabs(jet.momentum().rapidity()), weight );
	
	const double Yboost = fabs( boson_f.momentum().rapidity() +  jet.momentum().rapidity() )/2.;
	
	_h_bjet_Yboost->fill(Yboost, weight );
	
	if (ZpT > 20) {
	  
	  const double ZBDY   = fabs( boson_f.momentum().rapidity() -  jet.momentum().rapidity() );
	  const double ZBDPHI = fabs(deltaPhi(jet.momentum().phi(), boson_f.momentum().phi() ) );
	  const double ZBDR   = deltaR(jet.momentum().rapidity(), jet.momentum().phi(), 
				       boson_f.momentum().rapidity(), boson_f.momentum().phi() );
	  
	  _h_bjet_DY20   ->fill(ZBDY,   weight );
	  _h_bjet_ZdPhi20->fill(ZBDPHI, weight );
	  _h_bjet_ZdR20  ->fill(ZBDR,   weight );
	}
	
      } //loop over b-jets
      
      if (b_jets.size() < 2) return;
      
      _h_2bjet_ZPt->fill(ZpT, weight);
      _h_2bjet_ZY ->fill(ZY,  weight);
      
      const double BBDR =  deltaR(b_jets[0].momentum().rapidity(), b_jets[0].momentum().phi(), 
				  b_jets[1].momentum().rapidity(), b_jets[1].momentum().phi() );
      
      const double Ebb  = b_jets[0].momentum().E() + b_jets[1].momentum().E();
      const double Pxbb = b_jets[0].momentum().px() + b_jets[1].momentum().px();
      const double Pybb = b_jets[0].momentum().py() + b_jets[1].momentum().py();
      const double Pzbb = b_jets[0].momentum().pz() + b_jets[1].momentum().pz();	
      const double Mbb  = sqrt( fabs( Ebb*Ebb - Pxbb*Pxbb - Pybb*Pybb - Pzbb*Pzbb ) );  
      
      _h_2bjet_dR ->fill(BBDR, weight);
      _h_2bjet_Mbb->fill(Mbb,  weight);
      
    } // end of analysis loop
    
    
    /// Normalise histograms etc., after the run
    void finalize() {
      
      const double normfac = crossSection()/sumOfWeights();
      
      scale( _h_bjet_Pt, normfac);
      scale( _h_bjet_Y,  normfac);
      scale( _h_bjet_Yboost,  normfac);
      scale( _h_bjet_DY20,    normfac);
      scale( _h_bjet_ZdPhi20, normfac);
      scale( _h_bjet_ZdR20,   normfac);
      scale( _h_bjet_ZPt,  normfac);
      scale( _h_bjet_ZY,   normfac);
      scale( _h_2bjet_dR,  normfac);
      scale( _h_2bjet_Mbb, normfac);
      scale( _h_2bjet_ZPt, normfac);
      scale( _h_2bjet_ZY,  normfac);
    }
    
    //@}
    
    
  protected:
	  
    // Data members like post-cuts event weight counters go here
    size_t _mode;
    
    
  private:

    std::vector<PdgId> stableB;
    
    AIDA::IHistogram1D * _h_bjet_Pt;
    AIDA::IHistogram1D * _h_bjet_Y;
    AIDA::IHistogram1D * _h_bjet_Yboost;
    AIDA::IHistogram1D * _h_bjet_DY20;
    AIDA::IHistogram1D * _h_bjet_ZdPhi20;
    AIDA::IHistogram1D * _h_bjet_ZdR20;
    AIDA::IHistogram1D * _h_bjet_ZPt;
    AIDA::IHistogram1D * _h_bjet_ZY;
    AIDA::IHistogram1D * _h_2bjet_dR;
    AIDA::IHistogram1D * _h_2bjet_Mbb;
    AIDA::IHistogram1D * _h_2bjet_ZPt;
    AIDA::IHistogram1D * _h_2bjet_ZY;
    
  };


  class ATLAS_2014_I1306294_EL : public ATLAS_2014_I1306294 {
  public:
    ATLAS_2014_I1306294_EL()
      : ATLAS_2014_I1306294("ATLAS_2014_I1306294_EL")
    {
      _mode = 1;
    }
  };

  class ATLAS_2014_I1306294_MU : public ATLAS_2014_I1306294 {
  public:
    ATLAS_2014_I1306294_MU()
      : ATLAS_2014_I1306294("ATLAS_2014_I1306294_MU")
    {
      _mode = 2;
    }
  };


  // The hook for the plugin system
  DECLARE_RIVET_PLUGIN(ATLAS_2014_I1306294);
  DECLARE_RIVET_PLUGIN(ATLAS_2014_I1306294_MU);
  DECLARE_RIVET_PLUGIN(ATLAS_2014_I1306294_EL);

} 



